from DIRAC.ProductionSystem.Service.ProductionManagerHandler import (
    ProductionManagerHandler as DIRACProductionManagerHandler,
)


class ProductionManagerHandlerMixin:
    types_get_parent_transformation = [int]

    def export_get_parent_transformation(self, trans_id):
        res = self.productionDB.get_parent_transformation(trans_id)
        return res


class ProductionManagerHandler(
    ProductionManagerHandlerMixin, DIRACProductionManagerHandler
):
    pass



import DIRAC

from CTADIRAC.Interfaces.API.CTAJob import MetadataDict
from CTADIRAC.Interfaces.API.MCPipeJob import MCPipeJob


class MCSimTelProcessJob(MCPipeJob):
    """Generic MCPipe Job class"""

    def __init__(self) -> None:
        super().__init__(we_type="simtelprocessing")
        self.setType("DL0_Reprocessing")
        self.setName("SimTelProcessing")

    def set_output_metadata(self, metadata: MetadataDict = {}) -> None:
        super(MCPipeJob, self).set_output_metadata(metadata)

    def set_moon(self, moon="dark") -> None:
        """Set to simulate with various moon conditions
        Redefined from parent class because in the case of sim_telarray processing,
        we do not expect to run multiple NSBs in the same job.

        Parameters:
        moon -- moon conditions for simulation
        """

        moon_options = {
            "dark": ("", 1),
            "half": ("--with-half-moon", 5),
            "full": ("--with-full-moon", 19),
        }

        if isinstance(moon, list) or moon not in moon_options:
            moon_str = str(moon).replace("'", "")
            DIRAC.gLogger.error(
                f"Unknown moon option: {moon_str}. Options for simulation step are: \n dark \n half \n full "
            )
            DIRAC.exit(-1)
        else:
            DIRAC.gLogger.info(f"Set simulations with {moon} conditions")
            self.moon, self.output_file_metadata["nsb"] = moon_options[moon]

    def run_sim_telarray(self, debug=False) -> None:
        """
        Run sim_telarray processing of a CORSIKA file
        """
        prod_exe = "./dirac_sim_telarray_process"

        prod_args = ""

        cs_step = self.setExecutable(
            prod_exe,
            arguments=prod_args,
            logFile="Simtel_Log.txt",
            modulesList=["cta_script"],
        )
        cs_step["Value"]["name"] = "Step_Simtel"
        cs_step["Value"]["descr_short"] = "Run sim_telarray processing of CORSIKA file"

    def set_metadata_and_register_data(self) -> None:
        super().set_metadata_and_register_data(propagate_run_number=False)

    def run_dedicated_software(self) -> None:
        self.run_sim_telarray()

    def set_executable_sequence(self, debug: bool = False) -> None:
        super().set_executable_sequence(debug=debug, define_n_showers=False)



import json


class DMSEncoder(json.JSONEncoder):
    """This class is an encoder for the Requests, Operation and Files."""

    def default(self, obj):  # pylint: disable=method-hidden
        if hasattr(obj, "_getJSONData"):
            return obj._getJSONData()
        else:
            return json.JSONEncoder.default(self, obj)



import json

# # from DIRAC
from DIRAC import S_OK, S_ERROR

# from CTADIRAC
from CTADIRAC.DataManagementSystem.private.JSONUtils import DMSEncoder


class ProvBase:
    def toJSON(self):
        """Returns the JSON formated string that describes the agent"""
        try:
            jsonStr = json.dumps(self, cls=DMSEncoder)
            return S_OK(jsonStr)
        except Exception as e:
            return S_ERROR(str(e))

    def _jsonData(self, attrNames):
        """Returns the JSON formated string that describes the agent"""
        jsonData = {}
        for attrName in attrNames:
            # id might not be set since it is managed by SQLAlchemy
            if not hasattr(self, attrName):
                continue
            value = getattr(self, attrName)
            jsonData[attrName] = value

        return jsonData


class Activity(ProvBase):
    def __init__(
        self,
        id=None,
        name=None,
        startTime=None,
        endTime=None,
        comment=None,
        activityDescription_key=None,
    ):
        self.id = id
        self.name = name
        self.startTime = startTime
        self.endTime = endTime
        self.comment = comment
        self.activityDescription_key = activityDescription_key

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = [
            "id",
            "name",
            "startTime",
            "endTime",
            "comment",
            "activityDescription_key",
        ]
        return self._jsonData(attrNames)


class Entity(ProvBase):
    def __init__(
        self,
        id=None,
        classType=None,
        name=None,
        location=None,
        generatedAtTime=None,
        invalidatedAtTime=None,
        comment=None,
        entityDescription_key=None,
    ):
        self.id = id
        self.classType = classType
        self.name = name
        self.location = location
        self.generatedAtTime = generatedAtTime
        self.invalidatedAtTime = invalidatedAtTime
        self.comment = comment
        self.entityDescription_key = entityDescription_key

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = [
            "id",
            "classType",
            "name",
            "location",
            "generatedAtTime",
            "invalidatedAtTime",
            "comment",
            "entityDescription_key",
        ]
        return self._jsonData(attrNames)


class ValueEntity(Entity):
    def __init__(
        self,
        id=None,
        classType=None,
        name=None,
        location=None,
        generatedAtTime=None,
        invalidatedAtTime=None,
        comment=None,
        entityDescription_key=None,
        value=None,
    ):
        Entity.__init__(
            self,
            id=id,
            name=name,
            location=location,
            generatedAtTime=generatedAtTime,
            invalidatedAtTime=invalidatedAtTime,
            comment=comment,
            entityDescription_key=entityDescription_key,
            classType=classType,
        )
        self.value = value


class DatasetEntity(Entity):
    def __init__(
        self,
        id=None,
        classType=None,
        name=None,
        location=None,
        generatedAtTime=None,
        invalidatedAtTime=None,
        comment=None,
        entityDescription_key=None,
        ctadirac_guid=None,
    ):
        Entity.__init__(
            self,
            id,
            classType,
            name,
            location,
            generatedAtTime,
            invalidatedAtTime,
            comment,
            entityDescription_key,
        )
        self.ctadirac_guid = ctadirac_guid

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = [
            "id",
            "classType",
            "name",
            "location",
            "generatedAtTime",
            "invalidatedAtTime",
            "comment",
            "entityDescription_key",
            "value",
        ]
        return self._jsonData(attrNames)


class Used(ProvBase):
    def __init__(
        self,
        role=None,
        time=None,
        activity_key=None,
        entity_key=None,
        usageDescription_key=None,
    ):
        self.role = role
        self.time = time
        self.activity_key = activity_key
        self.entity_key = entity_key
        self.usageDescription_key = usageDescription_key

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = [
            "role",
            "time",
            "activity_key",
            "entity_key",
            "usageDescription_key",
        ]
        return self._jsonData(attrNames)


class WasGeneratedBy(ProvBase):
    def __init__(
        self,
        id=None,
        role=None,
        activity_key=None,
        entity_key=None,
        generationDescription_key=None,
    ):
        self.role = role
        self.activity_key = activity_key
        self.entity_key = entity_key
        self.generationDescription_key = generationDescription_key

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = ["role", "activity_key", "entity_key", "generationDescription_key"]
        return self._jsonData(attrNames)


class Agent(ProvBase):
    def __init__(
        self,
        id=None,
        name=None,
        type=None,
        email=None,
        comment=None,
        affiliation=None,
        phone=None,
        address=None,
        url=None,
    ):
        self.id = id
        self.name = name
        self.type = type
        self.email = email
        self.comment = comment
        self.affiliation = affiliation
        self.phone = phone
        self.address = address
        self.url = url

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = [
            "id",
            "name",
            "type",
            "email",
            "comment",
            "affiliation",
            "phone",
            "address",
            "url",
        ]
        return self._jsonData(attrNames)


class WasAttributedTo(ProvBase):
    def __init__(self, role=None, entity_key=None, agent_key=None):
        self.role = role
        self.entity_key = entity_key
        self.agent_key = agent_key

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = ["role", "entity_key", "agent_key"]
        return self._jsonData(attrNames)


class WasAssociatedWith(ProvBase):
    def __init__(self, role=None, activity_key=None, agent_key=None):
        self.role = role
        self.activity_key = activity_key
        self.agent_key = agent_key

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = ["role", "activity_key", "agent_key"]
        return self._jsonData(attrNames)


class ActivityDescription(ProvBase):
    def __init__(
        self,
        name=None,
        version=None,
        description=None,
        type=None,
        subtype=None,
        doculink=None,
    ):
        self.name = name
        self.version = version
        self.description = description
        self.type = type
        self.subtype = subtype
        self.doculink = doculink

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = ["name", "type", "subtype", "version", "doculink", "description"]
        return self._jsonData(attrNames)


class EntityDescription(ProvBase):
    def __init__(
        self, name=None, type=None, description=None, doculink=None, classType=None
    ):
        self.name = name
        self.type = type
        self.description = description
        self.doculink = doculink
        self.classType = classType

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = ["id", "name", "type", "description", "doculink", "classType"]
        return self._jsonData(attrNames)


class DatasetDescription(EntityDescription):
    def __init__(
        self,
        name=None,
        type=None,
        description=None,
        doculink=None,
        classType=None,
        contentType=None,
    ):
        EntityDescription.__init__(
            self,
            name=name,
            type=type,
            description=description,
            doculink=doculink,
            classType=classType,
        )
        self.contentType = contentType

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = ["name", "type", "description", "doculink", "contentType"]
        return self._jsonData(attrNames)


class ValueDescription(EntityDescription):
    def __init__(
        self,
        name=None,
        type=None,
        description=None,
        doculink=None,
        classType=None,
        valueType=None,
        unit=None,
        ucd=None,
        utype=None,
    ):
        EntityDescription.__init__(
            self,
            name=name,
            type=type,
            description=description,
            doculink=doculink,
            classType=classType,
        )
        self.valueType = valueType
        self.unit = unit
        self.ucd = ucd
        self.utype = utype

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = [
            "name",
            "type",
            "description",
            "doculink",
            "valueType",
            "unit",
            "ucd",
            "utype",
        ]
        return self._jsonData(attrNames)


class UsageDescription(ProvBase):
    def __init__(
        self,
        role=None,
        description=None,
        type=None,
        multiplicity=1,
        activityDescription_key=None,
        entityDescription_key=None,
    ):
        self.role = role
        self.description = description
        self.type = type
        self.multiplicity = multiplicity
        self.activityDescription_key = activityDescription_key
        self.entityDescription_key = entityDescription_key

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = [
            "role",
            "description",
            "type",
            "multiplicity",
            "activityDescription_key",
            "entityDescription_key",
        ]
        return self._jsonData(attrNames)


class GenerationDescription(ProvBase):
    def __init__(
        self,
        role=None,
        description=None,
        type=None,
        multiplicity=1,
        activityDescription_key=None,
        entityDescription_key=None,
    ):
        self.role = role
        self.description = description
        self.type = type
        self.multiplicity = multiplicity
        self.activityDescription_key = activityDescription_key
        self.entityDescription_key = entityDescription_key

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = [
            "role",
            "description",
            "type",
            "multiplicity",
            "activityDescription_key",
            "entityDescription_key",
        ]
        return self._jsonData(attrNames)


class WasConfiguredBy(ProvBase):
    def __init__(
        self,
        id=None,
        artefactType="Parameter",
        activity_key=None,
        parameter_key=None,
        configFile_key=None,
    ):
        self.id = id
        self.artefactType = artefactType
        self.activity_key = activity_key
        self.parameter_key = parameter_key
        self.configFile_key = configFile_key

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = [
            "id",
            "artefactType",
            "activity_key",
            "parameter_key",
            "configFile_key",
        ]
        return self._jsonData(attrNames)


class Parameter(ProvBase):
    def __init__(self, id=None, value=None, name=None, parameterDescription_key=None):
        self.id = id
        self.value = value
        self.name = name
        self.parameterDescription_key = parameterDescription_key

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = ["id", "value", "name", "parameterDescription_key"]
        return self._jsonData(attrNames)


class ConfigFile(ProvBase):
    def __init__(
        self, name=None, location=None, comment=None, configFileDescription_key=None
    ):
        self.name = name
        self.location = location
        self.comment = comment
        self.configFileDescription_key = configFileDescription_key

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = ["name", "location", "comment", "configFileDescription_key"]
        return self._jsonData(attrNames)


class ParameterDescription(ProvBase):
    def __init__(
        self,
        name=None,
        valueType=None,
        description=None,
        unit=None,
        ucd=None,
        utype=None,
        min=None,
        max=None,
        default=None,
        options=None,
        activityDescription_key=None,
    ):
        self.name = name
        self.valueType = valueType
        self.description = description
        self.unit = unit
        self.ucd = ucd
        self.utype = utype
        self.min = min
        self.max = max
        self.default = default
        self.options = options
        self.activityDescription_key = activityDescription_key

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = [
            "name",
            "valueType",
            "description",
            "unit",
            "ucd",
            "utype",
            "min",
            "max",
            "default",
            "options",
            "activityDescription_key",
        ]
        return self._jsonData(attrNames)


class ConfigFileDescription(ProvBase):
    def __init__(
        self,
        id=None,
        name=None,
        contentType=None,
        description=None,
        activityDescription_key=None,
    ):
        self.id = id
        self.name = name
        self.contentType = contentType
        self.description = description
        self.activityDescription_key = activityDescription_key

    def _getJSONData(self):
        """Returns the data that have to be serialized by JSON"""

        attrNames = ["name", "contentType", "description", "activityDescription_key"]
        return self._jsonData(attrNames)



#!/usr/bin/env python

__RCSID__ = "$Id$"

# generic imports
import os
import subprocess

# DIRAC imports
from DIRAC.Core.Base.Script import Script
import DIRAC
from DIRAC import gLogger
from DIRAC.Interfaces.API.Dirac import Dirac
from CTADIRAC.Core.Utilities.tool_box import run_number_from_filename
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from DIRAC.Resources.Catalog.TSCatalogClient import TSCatalogClient

Script.parseCommandLine(ignoreErrors=True)

DIRAC.initialize()  # Initialize configuration

dirac = Dirac()
fcc = FileCatalogClient()
tcc = TSCatalogClient()


def get_osb(job_id):
    res = dirac.getOutputSandbox(job_id)
    if not res["OK"]:
        gLogger.error(res["Message"])


def get_jdl(job_id):
    res = dirac.getJobJDL(job_id)
    if not res["OK"]:
        gLogger.error(res["Message"])
        DIRAC.exit(1)
    input_lfn_list = res["Value"]["InputData"]
    return input_lfn_list


def parse_log(job_id, lfn_list):
    #awk_cmd = "awk -F '/' '{print $7}'"
    #file = subprocess.check_output(
    #    f"grep 'Caught unexpected exception' {job_id}/ctapipe_merge_Log.txt | {awk_cmd}", shell=True
    #)
    error_string = subprocess.check_output(
        f"grep 'Caught unexpected exception' {job_id}/ctapipe_merge_Log.txt", shell=True
    )
    error_string = error_string.decode("utf-8").strip()
    file_name = error_string.split("/")[-1]
    gLogger.notice(f"corrupted file {file_name}")
    for lfn in lfn_list:
        if file_name in lfn:
            return lfn


def get_metadata(lfn):
    res = fcc.getFileUserMetadata(lfn)
    if not res["OK"]:
        gLogger.error(res["Message"])
        DIRAC.exit(1)
    job_id = res["Value"]["jobID"]
    return job_id


def get_corrupted_simtel_lfn(lfn_list, corrupted_lfn):
    file_name = os.path.basename(corrupted_lfn)
    run_number = f"run{run_number_from_filename(file_name, 'ctapipe')}"
    for lfn in lfn_list:
        if run_number in lfn:
            return lfn


def remove(lfn):
    gLogger.notice(f"removing {lfn}")
    res = fcc.removeFile(lfn)
    if not res["OK"]:
        gLogger.error(res["Message"])
    res = tcc.removeFile(lfn)
    if not res["OK"]:
        gLogger.error(res["Message"])


def main():
    argss = Script.getPositionalArgs()
    job_ids = argss
    for job_id in job_ids:
        get_osb(job_id)
        input_dl2_lfn_list = get_jdl(job_id)
        input_corrupted_dl2_lfn = parse_log(job_id, input_dl2_lfn_list)
        dl2_job_id = get_metadata(input_corrupted_dl2_lfn)
        input_simtel_lfn_list = get_jdl(dl2_job_id)
        input_corrupted_simtel_lfn = get_corrupted_simtel_lfn(
            input_simtel_lfn_list, input_corrupted_dl2_lfn
        )
        print(input_corrupted_simtel_lfn)
        #remove(input_corrupted_simtel_lfn)
        remove(input_corrupted_dl2_lfn)


if __name__ == "__main__":
    main()



""" Test definitions for submission and monitoring of jobs, transformations and productions
"""
import unittest
import subprocess as sp
import os
from glob import glob

import DIRAC
from DIRAC.Interfaces.API.Dirac import Dirac
from DIRAC.Resources.Catalog.FileCatalog import FileCatalog

DIRAC.initialize()  # Initialize configuration

from testJobDefinitions import (
    hello_world,
    hello_world_site_spec,
    mp_job,
    mandelbrot_simulation,
)
from testProdDefinitions import (
    create_prod_step1,
    clean_prod,
    start_prod,
    create_prod,
    get_prod_transformations,
)
from testTransDefinitions import (
    create_mc_transformation,
    get_trans,
    clean_trans,
    extend_trans,
    check_trans_status,
)
from testDMSDefinitions import (
    ClientDMS,
    DMSmetadata,
)
from testFileCatalogDefinitions import ClientFileCatalog

dirac = Dirac()

"""To add parser and running or skipping test using unittest
see: https://docs.python.org/3/library/unittest.html#skipping-tests-and-expected-failures
"""


class WMSTestCase(unittest.TestCase):
    """Base class for the Regression test cases"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.init_var()

    def init_var(
        self,
        sites=None,
        s_elements=None,
        test_dir="/vo.cta.in2p3.fr/tests/wms",
        input_file="WMS_TestInputFile.txt",
        output_file="WMS_TestOutputFile.txt",
        exe_script="exe_script.sh",
        trans_name="MCTransTest",
        prod_name="SeqProdTest",
        output_se="CSCS-Disk",
        prod_config="config/full_production_config.yml",
        n_tasks=21,
        test_to_skip=None,
    ):
        if not s_elements:
            self.s_elements = ["CSCS-Disk"]
        else:
            self.s_elements = s_elements
        if not sites:
            self.sites = [
                "CTAO.DESY-ZEUTHEN.de",
                "CTAO.PIC.es",
                "CTAO.CSCS.ch",
                "CTAO.FRASCATI.it",
            ]
        else:
            self.sites = sites
        self.test_dir = test_dir
        self.input_file = input_file
        self.output_file = output_file
        self.exe_script = exe_script
        self.inputLFN = os.path.join(self.test_dir, self.input_file)
        self.jobsSubmittedList = []
        self.trans_name = trans_name
        self.prod_name = prod_name
        self.output_se = output_se
        for dir, _, _ in os.walk(os.getcwd()):
            if glob(os.path.join(dir, prod_config)):
                self.prodConfigFile = os.path.join(dir, prod_config)
        self.n_tasks = n_tasks
        self.separation = (
            "----------------------------------------------------------------------"
        )
        self.test_to_skip = test_to_skip

    def setUp(
        self,
    ):  # to initialize the execution context. Called before the execution of each test method
        # a check on proxy for example can be added here
        pass

    def tearDown(
        self,
    ):  # called after the execution of each test method. To clean (close files, etc).
        pass


class JobExecutionTests(WMSTestCase):
    """submit jobs"""

    def test_submit_hello(self):
        """submit a hello world job, check status and logs"""
        print("\n")
        res = hello_world()
        self.assertTrue(res["OK"])
        # get job status
        jobid = res["Value"]
        res = Dirac().getJobStatus(jobid)
        self.assertTrue(res["OK"])
        # get logging informations
        res = Dirac().getJobLoggingInfo(jobid)
        self.assertTrue(res["OK"])

    def test_submit_sites(self):
        """submit a hello world job to a specific site"""
        print("\n")
        for site in self.sites:
            with self.subTest(i=site):
                res = hello_world_site_spec(site=site)
                self.assertTrue(res["OK"])

    def test_submit_mp(self):
        """submit a job requiring multiple cores"""
        print("\n")
        res = mp_job()
        self.assertTrue(res["OK"])

    def test_submit_mandelbrot_sim(self):
        """submit a mandelbrot simulation job"""
        print("\n")
        res = mandelbrot_simulation()
        self.assertTrue(res["OK"])


class TransformationExecutionTests(WMSTestCase):
    def test01_submit_transformation(self):
        print("\n")
        print(self.separation)
        res = get_trans(self.trans_name)
        if res["OK"]:
            print(f"Clean existing transformation {self.trans_name}")
            if res["Value"]["TransformationName"] == self.trans_name:
                res = clean_trans(self.trans_name)
                self.assertTrue(res["OK"])

        # create a MC transformation
        print("Create transformation")
        res = create_mc_transformation(self.trans_name)[0]
        self.assertTrue(res["OK"])

    def test02_extend_transformation(self):
        print("\n")
        print(self.separation)
        # extend the transformation
        print(f"Extend transformation {self.trans_name}")
        res = extend_trans(self.trans_name, self.n_tasks)
        self.assertTrue(res["OK"])

    def test03_monitor_transformation(self):
        print("\n")
        print(self.separation)
        # check the status of the transformation
        print(f"Check status of transformation {self.trans_name}")
        res = check_trans_status(self.trans_name)
        self.assertTrue(res["OK"])


class ProductionConfigurationTests(WMSTestCase):
    def test01_configure_production(self):
        print("\n")
        print(self.separation)
        prod_name = "prodTestFullProduction"
        res = sp.run(
            [f"cta-prod-submit {prod_name} {self.prodConfigFile} dry-run"],
            shell=True,
            check=True,
        )
        self.assertEqual(res.returncode, 0)


class ProductionExecutionTests(WMSTestCase):
    def test01_submit_production(self):
        print("\n")
        print(self.separation)
        print("Create Prod Step 1")
        res = create_prod_step1()[0]
        self.assertTrue(res["OK"])

        print(self.separation)
        print(f"Clean existing prod with name {self.prod_name}")
        res = clean_prod(prod_name=self.prod_name)
        self.assertTrue(res["OK"])

        print(self.separation)
        print(f"Create production {self.prod_name}")
        res = create_prod(prod_name=self.prod_name)
        self.assertTrue(res["OK"])

        print(self.separation)
        print(f"Start production {self.prod_name}")
        res = start_prod(prod_name=self.prod_name)
        self.assertTrue(res["OK"])

        print(self.separation)
        trans_list = get_prod_transformations(self.prod_name)
        for trans in trans_list:
            trans_id = trans["TransformationID"]
            print(f"Extend transformation {trans_id}")
            res = extend_trans(trans_id, n_tasks=self.n_tasks)
            self.assertTrue(res["OK"])

    def test02_monitor_production(self):
        print("\n")
        print(self.separation)
        res = sp.run(
            [f"dirac-prod-get {self.prod_name}"],
            shell=True,
            check=True,
        )
        self.assertEqual(res.returncode, 0)

    def test03_monitor_production(self):
        print("\n")
        print(self.separation)
        res = sp.run(
            [f"dirac-prod-get-trans {self.prod_name}"],
            shell=True,
            check=True,
        )
        self.assertEqual(res.returncode, 0)


class ProdSystemFullTests(WMSTestCase):
    """submit a full production composed of MC simulation, processing and 2 steps of merging"""

    def test_submit_full_prod(self):
        print("\n")
        print(self.separation)
        test_dir = "/vo.cta.in2p3.fr/tests/prodsys/MC"
        prod_name = "prodTestFullProduction"
        prod_test_dir = os.path.join(test_dir, "Prod5bTest/LaPalma/gamma-diffuse")

        # create test directory if not empty
        fc = FileCatalog()
        res = fc.createDirectory(prod_test_dir)
        if not res["OK"]:
            print(f"Failed to create directory {prod_test_dir}")
        self.assertTrue(res["OK"])
        # clean test directory
        res = sp.run(
            [f"dirac-dms-clean-directory {prod_test_dir}"], shell=True, check=True
        )
        self.assertEqual(res.returncode, 0)

        # create metadata fields
        dms_metadata = DMSmetadata()
        res = fc.createDirectory(dms_metadata.dataset_dir)
        self.assertTrue(res["OK"])
        res = dms_metadata.add_dir_md_in_dfc()
        self.assertTrue(res["OK"])
        res = dms_metadata.add_file_md_in_dfc()
        self.assertTrue(res["OK"])
        # clean an eventual existing production
        res = clean_prod(prod_name)
        self.assertTrue(res["OK"])
        res = sp.run(
            [f"cta-prod-submit {prod_name} {self.prodConfigFile}"],
            shell=True,
            check=True,
        )
        self.assertEqual(res.returncode, 0)

        trans_list = get_prod_transformations(prod_name)
        for trans in trans_list:
            trans_id = trans["TransformationID"]
            res = get_trans(trans_id)
            trans_type = res["Value"]["Type"]
            if trans_type == "MCSimulation":
                print(f"Extend transformation {trans_id}")
                res = extend_trans(trans_id=trans_id, n_tasks=10)
                self.assertTrue(res["OK"])


class ClientDMSDatasetTests(WMSTestCase):
    """Test Client DMS using dataset"""

    def dirac_dms_metadata(self):
        print("\n")
        print(self.separation)
        dms_md = DMSmetadata()
        test_cases = dms_md.main()

        for tests in test_cases.keys():
            with self.subTest(i=tests):
                for res in test_cases[tests]():
                    if isinstance(res, dict):
                        self.assertTrue(res["OK"])
                    elif isinstance(res, bool):
                        self.assertTrue(res)

    def cta_prod_dataset(self):
        """Test dataset command line client"""
        print("\n")
        print(self.separation)
        test_dataset = "Prod5b_LaPalma_AdvancedBaseline_NSB1x_electron_North_20deg_R1"
        test_dataset_json = "datasets/Prod5b_LaPalma_AdvancedBaseline_NSB1x_electron_North_20deg_R1.json"

        commands = [
            f"cta-prod-add-dataset {test_dataset_json}",
            f"cta-prod-update-dataset {test_dataset}",
            f"cta-prod-show-dataset {test_dataset} -l",
        ]

        for cmd in commands:
            with self.subTest(i=cmd):
                res = sp.getstatusoutput(cmd)
                print(res[1])
                self.assertEqual(res[0], 0)

    def test_dms_client_dataset(self):
        self.dirac_dms_metadata()
        self.cta_prod_dataset()


class ClientDMSTests(WMSTestCase):
    """test data management"""

    def test_client_dms(self):
        dms = ClientDMS()
        dms.write_test_file()
        print("====== Using SE", dms.dest_se1, dms.dest_se2)
        print("\n")
        print("====== Using test LFN", dms.test_lfn)
        print(dms.test_dir)
        res = sp.run(
            [f"set -x; dirac-dms-clean-directory {dms.test_dir}"],
            shell=True,
            check=True,
        )
        self.assertEqual(res.returncode, 0)
        res = sp.run(
            [
                f"set -x; dirac-dms-add-file {dms.test_lfn} ./{dms.test_file} {dms.dest_se1}"
            ],
            shell=True,
            check=True,
        )
        self.assertEqual(res.returncode, 0)
        sp.run([f"mv {dms.test_file} {dms.test_file}.old"], shell=True, check=True)
        res = sp.run(
            [f"set -x; dirac-dms-replicate-lfn  {dms.test_lfn} {dms.dest_se2}"],
            shell=True,
            check=True,
        )
        self.assertEqual(res.returncode, 0)
        res = sp.run(
            [f"set -x; dirac-dms-catalog-metadata {dms.test_lfn}"],
            shell=True,
            check=True,
        )
        self.assertEqual(res.returncode, 0)
        res = sp.run(
            [f"set -x; dirac-dms-lfn-metadata {dms.test_lfn}"], shell=True, check=True
        )
        self.assertEqual(res.returncode, 0)
        res = sp.run(
            [f"set -x; dirac-dms-lfn-accessURL {dms.test_lfn} {dms.dest_se1}"],
            shell=True,
            check=True,
        )
        self.assertEqual(res.returncode, 0)
        res = sp.run(
            [f"set -x; dirac-dms-get-file {dms.test_lfn}"], shell=True, check=True
        )
        self.assertEqual(res.returncode, 0)
        res = sp.run([f"ls {dms.test_file}"], shell=True, check=True)
        self.assertEqual(res.returncode, 0)
        if res.returncode:
            print("File downloaded properly")
        res = sp.run(
            [f"set -x; dirac-dms-lfn-replicas {dms.test_lfn}"], shell=True, check=True
        )
        self.assertEqual(res.returncode, 0)
        res = sp.run(
            [f"set -x; dirac-dms-remove-replicas {dms.test_lfn} {dms.dest_se2}"],
            shell=True,
            check=True,
        )
        self.assertEqual(res.returncode, 0)
        res = sp.run(
            [f"set -x; dirac-dms-lfn-replicas {dms.test_lfn}"], shell=True, check=True
        )
        self.assertEqual(res.returncode, 0)
        res = sp.run(
            [f"set -x; dirac-dms-remove-files {dms.test_lfn}"], shell=True, check=True
        )
        self.assertEqual(res.returncode, 0)
        res = sp.run(
            [f"set -x; dirac-dms-lfn-replicas {dms.test_lfn}"], shell=True, check=True
        )
        self.assertEqual(res.returncode, 0)
        sp.run([f"rm {dms.test_file}"], shell=True, check=True)
        sp.run([f"rm {dms.test_file}.old"], shell=True, check=True)


class ClientRucioFileCatalogTests(WMSTestCase):
    """test Rucio file catalog"""

    def test_client_file_catalog(self):
        fc = ClientFileCatalog("RucioFileCatalog")
        fc.write_test_file()

        res = fc.create_directory()
        self.assertTrue(res["OK"])

        res = fc.list_directory()
        self.assertTrue(res["OK"])

        res = fc.register_file()
        self.assertTrue(res["OK"])

        res = fc.put_and_register()
        self.assertTrue(res["OK"])

        res = fc.remove_file()
        self.assertTrue(res["OK"])


class ClientDIRACFileCatalogTests(WMSTestCase):
    """test DIRAC file catalog"""

    def test_client_file_catalog(self):
        fc = ClientFileCatalog("DIRACFileCatalog")
        fc.write_test_file()

        res = fc.create_directory()
        self.assertTrue(res["OK"])

        res = fc.list_directory()
        self.assertTrue(res["OK"])

        res = fc.register_file()
        self.assertTrue(res["OK"])

        res = fc.remove_file()
        self.assertTrue(res["OK"])

        res = fc.put_and_register()
        self.assertTrue(res["OK"])

        res = fc.remove_file()
        self.assertTrue(res["OK"])



from copy import deepcopy
from CTADIRAC.Interfaces.API.CTAJob import CTAJob, MetadataDict


class CtapipeApplyModelsJob(CTAJob):
    """Job extension class for ctapipe apply models"""

    def __init__(self) -> None:
        super().__init__(we_type="ctapipeapplymodels")
        self.setName("ctapipe_apply-models")
        self.setType("ApplyModel")
        self.prog_name = "ctapipe-apply-models"
        self.data_level = 2

    def run_ctapipe_apply_models(self) -> None:
        model_metadata: MetadataDict = deepcopy(self.output_metadata)
        model_metadata["outputType"] = "Model"
        model_metadata.update(self.output_file_metadata)
        model_metadata.pop("particle")
        if "split" in model_metadata:
            model_metadata.pop("split")
        if "energy_model.pkl" in self.options:
            model_metadata["analysis_prog"] = "ctapipe-train-energy-regressor"
            argument_list = []
            for key, value in model_metadata.items():
                argument_list.append(key + "=" + str(value))
            get_step0 = self.setExecutable(
                "cta-prod-get-file-by-query",
                arguments=f"{' '.join(argument_list)}",
                logFile="DownloadEnergyModel_Log.txt",
            )
            get_step0["Value"]["name"] = "Step_DownloadEnergyModel"
            get_step0["Value"]["descr_short"] = "Download Energy Model"

        if "classifier_model.pkl" in self.options:
            model_metadata["analysis_prog"] = "ctapipe-train-particle-classifier"
            argument_list = []
            for key, value in model_metadata.items():
                argument_list.append(key + "=" + str(value))
            get_step1 = self.setExecutable(
                "cta-prod-get-file-by-query",
                arguments=f"{' '.join(argument_list)}",
                logFile="DownloadClassifierModel_Log.txt",
            )
            get_step1["Value"]["name"] = "Step_DownloadClassifierModel"
            get_step1["Value"]["descr_short"] = "Download Classifier Model"

        step = self.setExecutable(
            "./dirac_ctapipe-apply-models_wrapper",
            arguments=f"{self.options}",
            logFile="ctapipe_apply_models_Log.txt",
            modulesList=["cta_script"],
        )
        step["Value"]["name"] = "Step_ctapipe_apply_models"
        step["Value"]["descr_short"] = "Run ctapipe apply models"

    def run_dedicated_software(self) -> None:
        self.run_ctapipe_apply_models()



"""
    Class to handle a workflow described in CWL
"""

__RCSID__ = "$Id$"

import os
import schema_salad
from cwltool.context import LoadingContext
from cwltool.load_tool import load_tool
import cwltool.process
from CTADIRAC.ProductionSystem.CWL.CWLWorkflowStep import WorkflowStep
from CTADIRAC.ProductionSystem.CWL.CWLUtils import get_source_name, topological_sort


class Workflow:
    """Class for workflow made of one or more workflow steps"""

    #############################################################################

    def __init__(self):
        """Constructor"""
        self.ordered_steps = []
        self.process = None
        self.inputs = None
        self.ext_inputs = {}
        self.sources = {}
        self.sources_outputs = {}

    def load(self, input_cwl, input_yaml):
        uri = "file://" + os.path.abspath(input_yaml)
        loader = schema_salad.ref_resolver.Loader(
            {"@base": uri, "path": {"@type": "@id"}}
        )
        loading_context = LoadingContext({"strict": False, "debug": True})
        self.inputs, _ = loader.resolve_ref(uri)
        self.process = load_tool(input_cwl, loading_context)

    def get_ext_inputs(self):
        # Match each workflow input with the corresponding value in the yaml input file
        for inp in self.process.tool["inputs"]:
            self.ext_inputs[inp["id"]] = self.inputs[
                cwltool.process.shortname(inp["id"])
            ]

    def get_steps_order_and_sources(self):
        deps = []
        for step in self.process.steps:
            self.sources[step] = []
            self.sources_outputs[step] = []
            parent_steps_id = set()
            for inp in step.tool["inputs"]:
                # If the input is created in another step:
                if "source" in inp and inp["source"] not in self.ext_inputs:
                    self.sources[step].append(inp)
                    parent_step_id = get_source_name(inp["source"])
                    # Find step corresponding to the step id
                    for s in self.process.steps:
                        if s.id == parent_step_id:
                            parent_step_id = s
                            for out in parent_step_id.tool["outputs"]:
                                self.sources_outputs[step].append(out)
                    parent_steps_id.add(parent_step_id)
            deps.append([step, parent_steps_id])

        for step in topological_sort(deps):
            self.ordered_steps.append(step)

    def run_workflow(self, input_yaml):
        cmd_list = []
        self.get_ext_inputs()
        self.get_steps_order_and_sources()
        for ordered_step in self.ordered_steps:
            step = WorkflowStep()
            step.process = ordered_step
            step.sources = self.sources[ordered_step]
            step.sources_outputs = self.sources_outputs[ordered_step]
            input_cwl = step.process.tool["run"].replace("file://", "")
            step.load(
                input_cwl,
                input_yaml,
                sources=step.sources,
                sources_outputs=step.sources_outputs,
            )
            step = step.get_command_line(
                sources=step.sources, sources_outputs=step.sources_outputs
            )
            cmd_list.append(step.command_line)
        return cmd_list



#!/usr/bin/env python


from DIRAC.Core.Base import Script

# Script.setUsageMessage('\n'.join([__doc__.split('\n')[1]]))

Script.parseCommandLine()

import DIRAC

# from CTADIRAC

from CTADIRAC.DataManagementSystem.Client.ProvClient import ProvClient
from CTADIRAC.DataManagementSystem.Client.ProvBase import Agent
from CTADIRAC.DataManagementSystem.Client.ProvBase import ActivityDescription
from CTADIRAC.DataManagementSystem.Client.ProvBase import DatasetDescription
from CTADIRAC.DataManagementSystem.Client.ProvBase import UsageDescription
from CTADIRAC.DataManagementSystem.Client.ProvBase import GenerationDescription
from CTADIRAC.DataManagementSystem.Client.ProvBase import ValueDescription
from CTADIRAC.DataManagementSystem.Client.ProvBase import ParameterDescription

provClient = ProvClient()

# Add an ActivityDescription and get its internal_key

actDesc1 = ActivityDescription(
    name="ctapipe-display-muons",
    type="Reconstruction",
    subtype="",
    version="0.6.2",
    doculink="",
    description="Muon reconstruction",
)
#
res = provClient.addActivityDescription(actDesc1)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)
else:
    activityDescription_key = res["Value"]["internal_key"]

# Create the description of input entities
dataDesc1 = DatasetDescription(
    name="protons_events",
    description="proton file",
    classType="datasetDescription",
    type="data",
    contentType="application/octet-stream",
)
res = provClient.addDatasetDescription(dataDesc1)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)
else:
    entityDescription_key = res["Value"]["internal_key"]

usedDesc1 = UsageDescription(
    activityDescription_key=activityDescription_key,
    entityDescription_key=entityDescription_key,
    role="dl0.sub.evt",
    type="Main",
)
res = provClient.addUsageDescription(usedDesc1)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)

# Create the description of output entities
dataDesc2 = DatasetDescription(
    name="muons",
    description="muon file",
    classType="datasetDescription",
    type="data",
    contentType="application/octet-stream",
)
res = provClient.addDatasetDescription(dataDesc2)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)
else:
    entityDescription_key = res["Value"]["internal_key"]

wGBDesc1 = GenerationDescription(
    activityDescription_key=activityDescription_key,
    entityDescription_key=entityDescription_key,
    role="dl1.tel.evt.muon",
    type="Main",
)
res = provClient.addGenerationDescription(wGBDesc1)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)

valueDesc1 = ValueDescription(
    name="status", description="activity status", type="Quality", valueType="string"
)
res = provClient.addValueDescription(valueDesc1)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)
else:
    entityDescription_key = res["Value"]["internal_key"]

wGBDesc2 = GenerationDescription(
    activityDescription_key=activityDescription_key,
    entityDescription_key=entityDescription_key,
    role="status",
    type="Quality",
)
res = provClient.addGenerationDescription(wGBDesc2)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)

# Create description of configFiles and Parameters
paramDesc1 = ParameterDescription(
    name="events",
    valueType="string",
    description="name of the event file",
    ucd="meta.id",
    utype="ProvenanceDM.ParameterDescription",
    activityDescription_key=activityDescription_key,
)
res = provClient.addParameterDescription(paramDesc1)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)

paramDesc2 = ParameterDescription(
    name="outfile",
    valueType="string",
    description="name of the output file",
    ucd="meta.id",
    utype="ProvenanceDM.ParameterDescription",
    activityDescription_key=activityDescription_key,
)
res = provClient.addParameterDescription(paramDesc2)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)

# CTAO Agent
agent = Agent(id="CTAO")
agent.name = "CTA Observatory"
agent.type = "Organization"

res = provClient.addAgent(agent)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)



#!/usr/bin/env python
"""
Get summary informations of all productions
"""
import DIRAC
from DIRAC.Core.Base.Script import Script
import typer
from rich.table import Table
from rich.console import Console

from CTADIRAC.Core.Utilities.typer_callbacks import (
    PRODUCTION_FIELDS,
    cond_dict_callback,
)

script = Script()
script.localCfg.isParsed = True
script.parseCommandLine(ignoreErrors=True)


app = typer.Typer()
long: bool = typer.Option(False, "--long", help="Long output.")

cond_dict = typer.Option(
    None,
    "--cond",
    callback=cond_dict_callback,
    help="Condition dictionnary to select fields.",
)


def extract_user_name_from_dn(author_dn):
    return author_dn.split("CN=")[-1]


def fill_columns(prod, fields, long):
    columns = []
    for f in fields:
        if f == "AuthorDN" and not long:
            columns += [str(extract_user_name_from_dn(prod[f]))]
        else:
            columns += [str(prod[f])]
    return columns


@app.command()
def main(long: bool = long, cond_dict: str = cond_dict):
    from DIRAC.ProductionSystem.Client.ProductionClient import ProductionClient

    prod_client = ProductionClient()
    res = prod_client.getProductions(condDict=cond_dict)
    fields = PRODUCTION_FIELDS
    if not long:
        fields.remove("CreationDate")
    table = Table(*fields)
    console = Console()
    if res["OK"]:
        prod_list = res["Value"]
        if not isinstance(res["Value"], list):
            prod_list = [res["Value"]]
        for prod in prod_list:
            columns = fill_columns(prod, fields, long)
            table.add_row(*columns)
    else:
        DIRAC.gLogger.error(res["Message"])
        DIRAC.exit(-1)

    console.print(table)


if __name__ == "__main__":
    app()



###############################################################################
# (c) Copyright 2022 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
""" Tornado-based HTTPs TransformationManager service.
"""
from DIRAC import gLogger
from DIRAC.TransformationSystem.Service.TornadoTransformationManagerHandler import (
    TornadoTransformationManagerHandler as TornadoTManagerBase,
)
from CTADIRAC.TransformationSystem.Service.TransformationManagerHandler import (
    TransformationManagerHandlerMixin,
)


sLog = gLogger.getSubLogger(__name__)


class TornadoTransformationManagerHandler(
    TransformationManagerHandlerMixin, TornadoTManagerBase
):
    log = sLog
